@@ -29,14 +29,16 @@ BEGIN {
                     ARCH_DIR DOT_EXISTS AUTO LIB_AUTO_DIR ARCH_AUTO_DIR    
                     PERL_CORE IS_INTERNALS_OBJ IS_CODEREF CREATE_FILE_URI   
                     STRIP_GZ_SUFFIX BLIB_LIBDIR GET_XS_FILES INSTALLER_BUILD
-                    INSTALLER_MM ON_OLD_CYGWIN IS_FILE IS_DIR
+                    INSTALLER_MM ON_OLD_CYGWIN IS_FILE IS_DIR INSTALLER_SAMPLE
                     INSTALL_VIA_PACKAGE_MANAGER
                 ];
 }
 
 use constant INSTALLER_BUILD
-                            => 'build';
-use constant INSTALLER_MM   => 'makemaker';    
+                            => 'CPANPLUS::Dist::Build';
+use constant INSTALLER_MM   => 'CPANPLUS::Dist::MM';    
+use constant INSTALLER_SAMPLE   
+                            => 'CPANPLUS::Dist::Sample';
 use constant INSTALL_VIA_PACKAGE_MANAGER 
                             => sub { my $fmt = $_[0] or return;
                                      return 1 if $fmt ne INSTALLER_BUILD and
